@@ -80,7 +80,7 @@ def get_lineage_graph(x, size=False, user=False, synapses=False,
     networkx.DiGraph
 
     """
-    x = int(x)
+    x = np.int64(x)
 
     client = get_cave_client(dataset=dataset)
     G = client.chunkedgraph.get_lineage_graph(x, as_nx_graph=True)
@@ -141,7 +141,7 @@ def get_lineage_graph(x, size=False, user=False, synapses=False,
 
     if proofreading_status:
         from .annotations import get_annotations
-        nodes = np.array(list(G.nodes)).astype(np.int64)
+        nodes = np.array(list(G.nodes), dtype=np.int64)
         pr = get_annotations('proofreading_status_public_v1',
                              filter_in_dict=dict(valid_id=nodes))
         if len(pr):
@@ -791,20 +791,31 @@ def is_latest_root(id, dataset='production', **kwargs):
     session = requests.Session()
     token = get_chunkedgraph_secret()
     session.headers['Authorization'] = f"Bearer {token}"
-
     url = f'https://prod.flywire-daf.com/segmentation/api/v1/table/{dataset}/is_latest_roots?int64_as_str=1'
-    post = {'node_ids': id[not_zero].tolist()}
-    r = session.post(url, json=post)
 
-    r.raise_for_status()
+    batch_size = 100_000
+    with navis.config.tqdm(desc='Checking',
+                           total=not_zero.sum(),
+                           disable=not_zero.sum() <= batch_size,
+                           leave=False) as pbar:
+        for i in range(0, not_zero.sum(), batch_size):
+            batch = id[not_zero][i:i+batch_size]
+            post = {'node_ids': batch.tolist()}
 
-    is_latest[not_zero] = np.array(r.json()['is_latest'])
+            # Update progress bar
+            pbar.update(len(batch))
+
+            r = session.post(url, json=post)
+
+            r.raise_for_status()
+
+            is_latest[np.where(not_zero)[0][i:i+batch_size]] = np.array(r.json()['is_latest'])
 
     return is_latest
 
 
 def update_ids(id,
-               sample=0.1,
+               stop_layer=2,
                supervoxels=None,
                dataset='production',
                progress=True, **kwargs):
@@ -815,10 +826,9 @@ def update_ids(id,
         2. If supervoxel provided, use it to update ID. Else try 3.
         3. See if we can map outdated IDs to a single up-to-date root (works
            if neuron has only seen merges). Else try 4.
-        4. For uncertain IDs, fetch all supervoxels and pick a random sample
-           (see ``sample`` parameter). Fetching the most recent root IDs for
-           the sample of supervoxels and return the root ID that was hit the
-           most often.
+        4. For uncertain IDs, fetch L2 IDs for the old root ID and the new
+           candidates. Pick the candidate containing most of the original L2
+           IDs.
 
     Parameters
     ----------
@@ -826,12 +836,14 @@ def update_ids(id,
                     Single ID or list of FlyWire (root) IDs. If DataFrame must
                     contain either a `root_id` or `root` column and optionally
                     a `supervoxel_id` or `supervoxel` column.
-    sample :        int | float
-                    Number (>= 1) or fraction (< 1) of super voxels to sample
-                    to guess the most recent version.
+    stop_layer :    int
+                    In case of root IDs that have been split, we need to
+                    determine the most likely successor. By default we do that
+                    using L2 IDs but you can speed this up by increasing the
+                    stop layer.
     supervoxels :   int | list-like, optional
                     If provided will use these supervoxels to update ``id``
-                    instead of sampling across all supervoxels.
+                    instead of sampling using the L2 IDs.
     dataset :       str | CloudVolume
                     Against which FlyWire dataset to query:
                       - "production" (current production dataset, fly_v31)
@@ -861,7 +873,7 @@ def update_ids(id,
     0  720575940621039145  720575940621039145           1    False
 
     """
-    assert sample > 0, '`sample` must be > 0'
+    assert stop_layer > 0, '`stop_layer` must be > 0'
 
     # See if we already check if this was the latest root
     is_latest = kwargs.pop('is_latest', None)
@@ -891,7 +903,7 @@ def update_ids(id,
                               dataset=dataset,
                               is_latest=il,
                               supervoxels=None,
-                              sample=sample) for x, il, in navis.config.tqdm(zip(id, is_latest),
+                              stop_layer=stop_layer) for x, il, in navis.config.tqdm(zip(id, is_latest),
                                                                                desc='Updating',
                                                                                leave=False,
                                                                                total=len(id),
@@ -907,7 +919,7 @@ def update_ids(id,
             elif any(pd.isnull(id)):
                 raise ValueError('`id` must not contain `None`')
 
-            id = np.array(id).astype(np.int64)
+            id = np.array(id, dtype=np.int64)
 
             res = pd.DataFrame()
             res['old_id'] = id
@@ -919,7 +931,7 @@ def update_ids(id,
         return res
 
     try:
-        id = int(id)
+        id = np.uint64(id)
     except ValueError:
         raise ValueError(f'"{id} does not look like a valid root ID.')
 
@@ -941,55 +953,38 @@ def update_ids(id,
             conf = 1
         elif supervoxels:
             try:
-                supervoxels = int(supervoxels)
+                supervoxels = np.int64(supervoxels)
             except ValueError:
                 raise ValueError(f'"{supervoxels} does not look like a valid '
                                  'supervoxel ID.')
             new_id = client.chunkedgraph.get_root_id(supervoxels_to_roots)
             conf = 1
         else:
-            # Get supervoxel ids - we need to use mip=0 because otherwise small
-            # neurons might not have any (visible) supervoxels
-            svoxels = roots_to_supervoxels(id, progress=False)[int(id)]
+            # Get L2 ids
+            # Note: we could also use higher level IDs
+            # (stop layer 3 or 4) which would be even fasters
+            l2_ids_orig = client.chunkedgraph.get_leaves(id, stop_layer=stop_layer)
+            l2_ids_new = [client.chunkedgraph.get_leaves(r, stop_layer=stop_layer) for r in pot_roots]
 
-            # Note: instead of supervoxels we could also use higher level IDs
-            # (stop layer 3 or 4) which might be much faster
-
-            # Shuffle voxels
-            np.random.shuffle(svoxels)
-
-            # Generate sample
-            if sample >= 1:
-                smpl = svoxels[: sample]
-            else:
-                smpl = svoxels[: max(int(len(svoxels) * sample), 1)]
-
-            # Fetch up-to-date root IDs for the sampled supervoxels
-            roots = supervoxels_to_roots(smpl, dataset=vol, progress=False)
-
-            # Find unique Ids and count them
-            unique, counts = np.unique(roots, return_counts=True)
+            # Get the fraction of original L2 IDs in each of the new root IDs
+            counts = np.array([np.isin(l2_ids_orig, ids).sum() for ids in l2_ids_new])
+            counts = counts / len(l2_ids_orig)
 
             # Get sorted indices
             sort_ix = np.argsort(counts)
 
             # New Id is the most frequent ID
-            new_id = unique[sort_ix[-1]]
+            new_id = pot_roots[sort_ix[-1]]
 
-            # Confidence is the difference between the top and the 2nd most
-            # frequent ID
-            if len(unique) > 1:
-                conf = round((counts[sort_ix[-1]] - counts[sort_ix[-2]]) / sum(counts),
-                             2)
-            else:
-                conf = 1
+            # Confidence is the fraction of original L2 IDs in the new ID
+            conf = round(counts[sort_ix[-1]], 2)
     else:
         new_id = id
         conf = 1
 
     return pd.DataFrame([[id, new_id, conf, id != new_id]],
                         columns=['old_id', 'new_id', 'confidence', 'changed']
-                        ).astype({'old_id': int, 'new_id': int})
+                        ).astype({'old_id': np.uint64, 'new_id': np.uint64})
 
 
 def snap_to_id(locs, id, snap_zero=False, dataset='production',
